#!/bin/env python3

import ctypes
from ctypes import *
from pydbus import SystemBus
import dbus
import dbus.service
import dbus.mainloop.glib
import syslog
from kdk_accessctl import kyaccessctl

import gi
gi.require_version('Gtk', '3.0') 
from gi.repository import GLib, Gtk, Gio

from dbus.mainloop.glib import DBusGMainLoop
DBusGMainLoop(set_as_default=True)

BUS_NAME = "com.kylin.kysdk.processsec"
PROC_IFACE_NAME = "com.kylin.kysdk.processsec.process"
KMOD_IFACE_NAME = "com.kylin.kysdk.processsec.kmod"
PROC_OBJ_NAME = "/com/kylin/kysdk/processsec/process"
KMOD_OBJ_NAME = "/com/kylin/kysdk/processsec/kmod"

KLOG_INFO = 6
KDK_MODULE_PROCESS = 7

class KdkProcess(dbus.service.Object):
    """
    <node>
        <interface name='com.kylin.kysdk.processsec.process'>
            <method name='SetAntiKilled' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='CancelAntiKilled' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetAntiKilled' >
                <arg type='as' direction='out' name='list'/>
            </method>
            <method name='SetExec' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='CancelExec' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>

        </interface>
    </node>
    """

    def __init__(self):
        try:
            self.libkyprocesssec = ctypes.CDLL("libkyprocesssec.so")
            self.liblogsec = ctypes.CDLL("libkylogsec.so")

            bus = dbus.SystemBus()
            bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
            dbus.service.Object.__init__(self, bus_name, PROC_OBJ_NAME)
            self.dbus = SystemBus().get("org.freedesktop.DBus", "/org/freedesktop/DBus")

        except Exception:
            exit(-1)

    @dbus.service.method(dbus_interface=PROC_IFACE_NAME, in_signature="s", out_signature="i", sender_keyword='sender')
    def SetAntiKilled(self, filepath, sender=None):
        uid = self.dbus.GetConnectionUnixUser(sender)
        pid = self.dbus.GetConnectionUnixProcessID(sender)
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("SetAntiKilled", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Call", encoding = "utf8")))

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                ctypes.c_char_p(bytes("SetAntiKilled", encoding = "utf8")),
                                ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        rc = self.libkyprocesssec.kdk_process_set_anti_killed(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("SetAntiKilled", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
        return rc

    @dbus.service.method(dbus_interface=PROC_IFACE_NAME, in_signature="s", out_signature="i", sender_keyword='sender')
    def CancelAntiKilled(self, filepath, sender=None):
        uid = self.dbus.GetConnectionUnixUser(sender)
        pid = self.dbus.GetConnectionUnixProcessID(sender)
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("CancelAntiKilled", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Call", encoding = "utf8")))
        
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("CancelAntiKilled", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        rc = self.libkyprocesssec.kdk_process_cancel_anti_killed(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("CancelAntiKilled", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
        return rc

    @dbus.service.method(dbus_interface=PROC_IFACE_NAME, in_signature="", out_signature="as", sender_keyword='sender')
    def GetAntiKilled(self, sender=None):
        uid = self.dbus.GetConnectionUnixUser(sender)
        pid = self.dbus.GetConnectionUnixProcessID(sender)
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("GetAntiKilled", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Call", encoding = "utf8")))
        out = []
        count = ctypes.c_int(0)
        
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("GetAntiKilled", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
            
            count = kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value
            return out

        self.libkyprocesssec.kdk_process_get_anti_killed.restype = ctypes.POINTER(ctypes.c_char_p)
        plist = self.libkyprocesssec.kdk_process_get_anti_killed(ctypes.byref(count))

        for p in range(0, count.value):
            out.append(str(plist[p], "UTF-8"))
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("GetAntiKilled", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
        return out

    @dbus.service.method(dbus_interface=PROC_IFACE_NAME, in_signature="s", out_signature="i", sender_keyword='sender')
    def SetExec(self, filepath, sender=None):
        uid = self.dbus.GetConnectionUnixUser(sender)
        pid = self.dbus.GetConnectionUnixProcessID(sender)
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("SetExec", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Call", encoding = "utf8")))
        
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("SetExec", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        rc = self.libkyprocesssec.kdk_process_set_executable(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("SetExec", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
        return rc

    @dbus.service.method(dbus_interface=PROC_IFACE_NAME, in_signature="s", out_signature="i", sender_keyword='sender')
    def CancelExec(self, filepath, sender=None):
        uid = self.dbus.GetConnectionUnixUser(sender)
        pid = self.dbus.GetConnectionUnixProcessID(sender)
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("CancelExec", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Call", encoding = "utf8")))
        
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("CancelExec", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        rc = self.libkyprocesssec.kdk_process_cancel_executable(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("CancelExec", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
        return rc

class KdkKmod(dbus.service.Object):
    """
    <node>
        <interface name='com.kylin.kysdk.processsec.kmod'>
            <method name='SetAntiUnloadedPath' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='CancelAntiUnloadedPath' >
                <arg type='s' direction='in' name='filepath'/>
                <arg type='i' direction='out' name='ret'/>
            </method>
            <method name='GetAntiUnloaded' >
                <arg type='as' direction='out' name='list'/>
            </method>
        </interface>
    </node>
    """

    def __init__(self):
        try:
            self.libkyprocesssec = ctypes.CDLL("libkyprocesssec.so")
            self.liblogsec = ctypes.CDLL("libkylogsec.so")
            self.libaccessctl = ctypes.CDLL("libkyaccessctl.so")

            bus = dbus.SystemBus()
            bus_name = dbus.service.BusName(BUS_NAME, bus=bus)
            dbus.service.Object.__init__(self, bus_name, KMOD_OBJ_NAME)
            self.dbus = SystemBus().get("org.freedesktop.DBus", "/org/freedesktop/DBus")

        except Exception:
            exit(-1)

    @dbus.service.method(dbus_interface=KMOD_IFACE_NAME, in_signature="s", out_signature="i", sender_keyword='sender')
    def SetAntiUnloadedPath(self, filepath, sender=None):
        uid = self.dbus.GetConnectionUnixUser(sender)
        pid = self.dbus.GetConnectionUnixProcessID(sender)
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("SetAntiUnloadedPath", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Call", encoding = "utf8")))

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("SetAntiUnloadedPath", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        rc = self.libkyprocesssec.kdk_kmod_set_anti_unloaded_by_path(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("SetAntiUnloadedPath", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
        return rc

    @dbus.service.method(dbus_interface=KMOD_IFACE_NAME, in_signature="s", out_signature="i", sender_keyword='sender')
    def CancelAntiUnloadedPath(self, filepath, sender=None):
        uid = self.dbus.GetConnectionUnixUser(sender)
        pid = self.dbus.GetConnectionUnixProcessID(sender)
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("CancelAntiUnloadedPath", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Call", encoding = "utf8")))
        
        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("CancelAntiUnloadedPath", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
            return kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value

        rc = self.libkyprocesssec.kdk_kmod_cancel_anti_unloaded_by_path(ctypes.c_char_p(bytes(filepath, encoding = "utf8")))
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("CancelAntiUnloadedPath", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
        return rc

    @dbus.service.method(dbus_interface=KMOD_IFACE_NAME, in_signature="", out_signature="as", sender_keyword='sender')
    def GetAntiUnloaded(self, sender=None):
        uid = self.dbus.GetConnectionUnixUser(sender)
        pid = self.dbus.GetConnectionUnixProcessID(sender)
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("GetAntiUnloaded", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Call", encoding = "utf8")))
        out = []
        count = ctypes.c_int(0)

        rc = kyaccessctl.KdkAccessCtl().kdk_accessctl_check_callable(sender, kyaccessctl.KdkModuleType.KDK_MODULE_PROCESS.value)
        if(rc == kyaccessctl.KdkAccessCtlStatus.KDK_ACCESSCTL_DISABLE.value):
            self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("GetAntiUnloaded", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
            count = kyaccessctl.KdkAccessCtlErrnoStatus.KDK_ERRNO_ACCESSCTL_DENY.value
            return out

        self.libkyprocesssec.kdk_kmod_get_anti_unloaded.restype = ctypes.POINTER(ctypes.c_char_p)
        plist = self.libkyprocesssec.kdk_kmod_get_anti_unloaded(ctypes.byref(count))

        for p in range(0, count.value):
            out.append(str(plist[p], "UTF-8"))
        self.liblogsec.kdk_sec_log(c_int(KLOG_INFO), c_int(KDK_MODULE_PROCESS), c_int(pid), c_int(uid),
                                   ctypes.c_char_p(bytes("GetAntiUnloaded", encoding = "utf8")),
                                   ctypes.c_char_p(bytes("Exit", encoding = "utf8")))
        return out
    

if __name__ == '__main__':
    loop = GLib.MainLoop()
    bus = dbus.SystemBus()
    bus.request_name(BUS_NAME)
    KdkProcess()
    KdkKmod()
    loop.run()